package me.eax.mostviewed

import scala.io._
import scala.util.matching.Regex._

object Application extends App {
  def buildStatUrl(domain: String, date: String) = {
    s"http://www.liveinternet.ru/stat/${domain}/" +
     s"pages.html?date=${date}&period=month&total=yes&per_page=100"
  }

  def fetchStatUrl(domain: String, date: String) = {
    val url = buildStatUrl(domain, date)
    Source.fromURL(url).mkString 
  }

  def getStat(domain: String, date: String) = {
    val data = fetchStatUrl(domain, date)
    val re = """(?s)for="id_\d+"><a href="([^"]+)"[^>]*>.*?<td>([\d,]+)</td>""".r
    for (re(url, count) <- re findAllMatchIn data) 
      yield (url, count.split("").filter(_ != ",").mkString.toInt)
  }

  def getTitle(url: String) = {
    val data = Source.fromURL(url).mkString
    val reStr = """<h2>(.*?)</h2>"""
    reStr.r findFirstMatchIn data match {
      case Some(x: Match) => x group 1
      case None => throw new RuntimeException(s"$url does not match $reStr")
    }
  }

  def numberOfViews(views: Int) = {
    val end = {
      val rem100 = views % 100
      if(5 <= rem100 && rem100 <= 20) "ов"
      else {
          val rem10 = views % 10
          if(rem10 == 1) ""
          else if(2 <= rem10 && rem10 <= 4) "а"
          else "ов"
        }
    }
    s"$views просмотр$end"
  }

  if(args.size < 3) {
    println("Usage: mostviewed <domain> <date> <number>")
    sys.exit(1)
  }

  val Array(domain, date, numberStr, _*) = args
  val number = numberStr.toInt

  println("<ul>")
  for((url, views) <- getStat(domain, date)
                        .filter(_._1 != s"http://$domain/")
                        .take(number)) {
    println(s"""<li><a href="${url}">${getTitle(url)}</a>, """ +
            s"${numberOfViews(views)} за месяц</li>")
  }
  println("</ul>")
}
